/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 2000
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/
#include "BrowseButton.h"
/* BrowseButton.m
 * Button to open a file selection dialog 
 * and *replace* text of delegate by selected file name.
 * 
 * See SXSettings.m for examples.
 */

#include <Foundation/NSObject.h>
#include <Foundation/NSString.h>
#include <GToolKit/GToolKit.h>
#include <stdio.h>

@implementation BrowseButton

+buttonWithDelegate: (GTKEditable*) editable_widget {
	return [self buttonWithLabel: @"" delegate: editable_widget];
}
+buttonWithLabel: (const NSString*) label delegate: (GTKEditable*) editable_widget{
	return [[[self alloc] initWithLabel: label delegate: editable_widget]
		 	autorelease];
}
-initWithDelegate: (GTKEditable*) editable_widget{
	return [self initWithLabel: @"" delegate: editable_widget];
}
-initWithLabel: (const NSString*) label delegate: (GTKEditable*) editable_widget{
	id res = [super initWithLabel: label];
	if( !res ) return NULL;
	delegate = [editable_widget retain];
	[self connectSignal:@"clicked" withTarget: self 
		sel: @selector(raiseFileDialog) ];
	filesel = NULL;
	return self;
}
// connect to delegate
-(void)setDelegate: (GTKEditable*) editable_widget{
	[delegate release];
	delegate = [editable_widget retain];
}
-(void) ok {
	gint pos = 0;
	NSString * path = [filesel getFilename];
	[GTKApp stopModalWithCode: YES];
	[delegate deleteText:0 endPos:-1]; 
	[delegate insertText:path newTextLength:[path length] position:&pos];
	[filesel destroy];
	filesel = NULL;
}
-(void) releaseFilesel {
//	[filesel release];
fprintf(stderr, "free FS\n");
	filesel = NULL;
}
-(void)raiseFileDialog {
	NSString * a_path  = [delegate getChars:0 endPos:-1]; 
	filesel = [[GTKFileSelection alloc]	initWithTitle: @"Select Path"];
	[[filesel okButton] connectSignal:@"clicked" withTarget:self
		sel:@selector(ok)];
	[[filesel cancelButton] connectSignal:@"clicked" withTarget:filesel
		sel:@selector(destroy)];
	[[filesel cancelButton] connectSignal:@"destroy" withTarget:self
		sel:@selector(releaseFilesel)];
	if( [a_path length] ) [filesel setFilename: a_path];
	[filesel setPosition: GTK_WIN_POS_CENTER];
	[filesel show];

	[GTKApp runModalForWindow:filesel];
}
-(void) dealloc {
	[delegate release];
	[super dealloc];
}
@end
