/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	Column.h - class to store columns of data with further information.
 *
 *	Andreas Beyer, 1998
 */
#ifndef Class_Column
#define Class_Column

#include <Foundation/NSObject.h>

typedef struct {
	double value;
	double upper_b;
	double lower_b;
} TRIPLE;

@interface Column : NSObject {
	char   *label;
	double *column;
	unsigned size;
	// statistical information:
	double	mean,		// mean
		conf,		// confidence intervall of the mean (95%)
		median,		// median
		med_conf_l,	// lower confidence intervall of the median (95%)
		med_conf_u,	// upper confidence intervall of the median (95%)
		quart_l,	// 25%% quartil
		quart_u,	// 75%% quartil
		std_var,	// standard variation
		stdv_mean,	// standard variation of the mean
		min, max;	// minimum & maximum
		
}
// make copy of array
-init: (const double*) array size: (unsigned) size label: (const char*) lb;
-initProperties;
-setLabel: (const char*) lb;
-(unsigned) count;
-(const char*) getLabel;
// return data at position 'pos'
-(double) dataAt: (unsigned) pos;
// return the ORIGINAL array of data
-(double*) getArray;
// returns YES if all data are either 0 or 1
-(BOOL) isBinary;
// returns YES if all numbers are positive integers
-(BOOL) hasPosIntegers;
// returns YES if condition is true for all data in the column:
-(BOOL) allEntriesLessOrEqual: (double) a_value;
-(BOOL) allEntriesLargerOrEqual: (double) a_value;
-(BOOL) allEntriesLessThan: (double) a_value;
-(BOOL) allEntriesLargerThan: (double) a_value;

//>>>> STATISTICAL INFORMATION
-(double) min;
-(double) max;
/* TRIPLE.value = mean, 
   TRIPLE.upper_b = mean + conf, 
   TRIPLE.lower_b = mean - conf
*/
-(TRIPLE) mean;
/* TRIPLE.value = median, 
   TRIPLE.upper_b = med_conf_u, 
   TRIPLE.lower_b = med_conf_l
*/
-(TRIPLE) median;
-(double) upperQuartil;
-(double) lowerQuartil;
-(double) stdVariation;
-(double) stdVarOfMean;

@end
#endif	// Class_Column
