/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	DataDisplay.h - display data in a table.
 *
 *
 *	Andreas Beyer, 1998
 */
#ifndef Class_DataDisplay
#define Class_DataDisplay

#include <Foundation/NSObject.h>
#include <Foundation/NSAutoreleasePool.h>
#include <GToolKit/GToolKit.h>
#include <GTKSheet/GTKSheet.h>

#include "Manager.h"
#include "DataSet.h"
#include "SXProgressBar.h"


// if this is defined, unselected cells will always get white background
#define CELLS_BG_WHITE

@interface DataDisplay : GTKSheet <ProgressDelegate> {
	DataSet	*ds;
	Manager	*manager;
	NSString	*title;
	
	BOOL		loading;
	BOOL		stop_loading; // If this is true initialization will be stoped
	NSMutableArray *selected_cols;
	int n_rows, n_cols;
	unsigned 	current_element; // used for progress bar
	double	total_n_elements;
	GdkColor	*bg_normal, *bg_selected, *fg_normal, *fg_selected;
	/* The index of the column in which the current popup menu appears.
	   Undefined state if no popup menu open. */
	unsigned	popup_target;
}
+dataDisplayWithDataSet: (DataSet*) data_set;
-initWithDataSet: (DataSet*) data_set;
-(void)setColors;
-(NSString*) getTitle;
-(DataSet*)getDataSet;
-(GTKWidget*)getTable;
-(void)selectColumnWithIndex: (unsigned) index;
-(void)unselectColumnWithIndex: (unsigned) index;
// select all Columns
-(void)selectAllCols;
-(void)unselectAllCols;
-(NSArray*)getSelectedCols;

// handle popup
-(void)buttonPressEvent:sender data: data;
-(void)raisePopupMenuForColumn: (unsigned) col event: (GdkEventButton*) event;
//-(void) buttonClicked: sender;
// handle select_column event:
-(void) doSelectColumn: sender;
-(void) doActivate;

-(void)displayStandardData;

// For SXProgressBar:
-(double) state;
-(void) stopOperation;	// Called when the user selects "Cancel"
@end
#endif // Class_DataDisplay
