/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	DataSet.h - store and evaluate data.
 *
 *
 *	Andreas Beyer, 1998
 */
#ifndef Class_DataSet
#define Class_DataSet

#include <stdlib.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSObject.h>

#include "Manager.h"
#include "SXProgressBar.h"

@class Column;
@class SXFileHandle;

@interface DataSet : NSObject <ProgressDelegate> {
	Manager		*manager;
	NSString	*fileName;
@private
	// Number of columns
	unsigned	n_cols;
	// Number of lines for each column. (May not be the same for all cols.)
	// Vector has length n_cols.
	unsigned	*n_lines;
/* Number of column containing the labels of the rows. The existence of the variable 
   'row_labels' Indicates whether 'col_with_row_labels' contains a valid number. */
	unsigned	col_with_row_labels; 
	NSMutableArray	*cols;// columns in a List
	NSMutableArray  *row_labels; // Labels of rows.
// The following variables are valid ONLY while loading a data file:
	NSString*	colSep;	// column separator in data file
	SXFileHandle 	*fh;		// FileHandle for access to data file
	double**	d_cols;	// temporarily used while reading files
	/* This is a list of columns that should be skiped when reading the file.
	   The array must contain ascending numbers of columns.
	   (e.g. skip_cols = {0, 3, 123}; would indicate to skip columns 0, 3
	    and 123)
	   skip_cols is initialized at the beginning of method -loadData.	
	   n_skip_cols contains the number of columns to be skiped. It
	   is thus equal to the length of 'skip_cols'. */
	unsigned	*skip_cols, n_skip_cols;
	unsigned	current_line; // while loading files: #of current line
	unsigned	total_lines;// total number of lines in current file
	BOOL		stop_operation; // stops loading a file
}
// for 'statist' *only*!
+(const char*) get_label: (double*) an_array;

//>>>>>> GENERAL
-initWithFile: (NSString*) file_name;
-(BOOL) readDataFromStdin;
-(BOOL) loadFile: (NSString*) pathName;
// This method assumes that the SXFileHandle fh exists and reads data from it.
-(BOOL) loadData;
//-saveFile;
//-saveFileAs: (NSString*) file_name;
//>>>>>> ProgressDelegate
-(double) state;	// return the current state in percent or -1
-(void)stopOperation;	// Called when the user selects "Cancel"

//>>>>>> INFORMATION
// name of loaded file
-(NSString*) getFileName;
// return number of columns
-(unsigned) countColumns;
// return number of rows in column number 'col'
-(unsigned) countRows: (unsigned) col;
-(Column*) getColumn: (unsigned) index;
-(unsigned) getIndexOfColumnWithLabel: (NSString*) label;
-(const char*) getLabelOfColumn: (unsigned) col;
// return data at position col:row
-(double) getDataAtColumn: (unsigned) col row: (unsigned) row;
// return NSArray containing the labels of the rows
-(NSArray*) getRowLabels;
-swapCols: (unsigned) col1 : (unsigned) col2;

//==========================================================================
//>>>>>> statistical stuff:

//==================== Standard Menu =======================================
// mean fo column 'col'
-(double) meanOfCol: (unsigned) col;

//>>>>>> DEBUGGING
// rather for debugging, than sufficient use:
-printStandardData: (unsigned) col;	// --> std.out
-printTable;				// --> std.out

@end
#endif // Class_DataSet
