/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/** FileList.m
 *
 *	Andreas Beyer, 1999
 */
#include <Foundation/NSString.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSException.h>
#include <GToolKit/GToolKit.h>

#include "FileList.h"
#include "Manager.h"

@interface FileListEntry: NSObject {
@private
	NSString* file_path;	// file path
	NSString* menu_path; // entry in the menu
}
-initWithPath: (NSString*) file_name;
-(NSString*) getFilePath;
-(NSString*) getMenuPath;
-(void)openFile;
@end
@implementation FileListEntry
-initWithPath: (NSString*) file_name {
	[super init];
	file_path = [file_name retain];
	menu_path = [NSString stringWithFormat: @"/File/%@",
						[file_path lastPathComponent]];
	return self;
}
-(NSString*) getFilePath {
	return file_path;
}
-(NSString*) getMenuPath {
	return menu_path;
}
-(void)openFile {
	[[Manager getManager] openFile: file_path];
}
-(void)dealloc {
	[file_path release];
	[menu_path release];
	[super dealloc];
}
@end

@implementation FileList
+fileListWithItemFactory:(GTKItemFactory*) _item_factory maxNFiles:(unsigned) n {
	return [(FileList*) [[self alloc] autorelease]
		initWithItemFactory: _item_factory maxNFiles: n];
}
-initWithItemFactory:(GTKItemFactory*) _item_factory maxNFiles:(unsigned) n {
	[super init];
	max_n_files = n;
	item_factory = _item_factory;
	lastFiles = [NSMutableArray new]; // Array of FileListEntries
	if(! lastFiles )
		[NSException raise: NSMallocException
			format: @"initWithItemFactory: Not enough memory."];
	return self;
}
-(void)addFileName:(NSString*) file_name {
	FileListEntry* entry;
	int i, n = [lastFiles count];
	GTKItemFactoryEntry menus[] = {
	   {{ NULL, NULL, NULL, 0, NULL }, @selector(openFile)},
	};
	if(! item_factory ) return;
	for(i = 0; i < n; i++) {
		if([file_name isEqualToString: [(FileListEntry*) 
					[lastFiles objectAtIndex: i] getFilePath] ])
			break;
	}
	if( i < n ) return; // file name is already in list
	if( n >= max_n_files ) { // remove last entry in the list
		[item_factory deleteItem: [(FileListEntry*) [lastFiles lastObject]
			 getMenuPath] ];
		[lastFiles removeLastObject];
	}
	entry = [[FileListEntry alloc] initWithPath: file_name];
	if(! entry )
		[NSException raise: NSMallocException
			format: @"initWithItemFactory: Not enough memory."];
	[lastFiles insertObject: entry atIndex: 0];
	menus[0].item.path = [[entry getMenuPath] cString];
	[item_factory createItems:&menus[0] count:1 forTarget: entry];
}
@end
