/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 2000
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	GPPrintDialog.m - Dialog, that requests parameters 
 *	for printing gnuplot plots.
 *
 *
 *	Andreas Beyer, 2000
 */
#include "GPPrintDialog.h"
#include "Message.h"

#include <Foundation/NSObject.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSException.h>
#include <Foundation/NSString.h>

#include <GToolKit/GToolKit.h>

@implementation GPPrintDialog
+dialog {
	return [ (GPPrintDialog*) [[self alloc] autorelease] init];
}

-init {
	GTKVBox * v_box;
	GTKHBox * buttons;
	GTKHBox * h_boxes[5];
	GTKLabel* label;
	unsigned i;
	[super init];
	// set default styles
	color_style = sx_color;
	line_style  = sx_solid;
	// make the two buttons
	ok_button = [GTKButton buttonWithLabel: @"OK"];
	[ok_button setUsize: 100 height: 30];
	[ok_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithTrue)]];
	cancel_button = [GTKButton buttonWithLabel: @"Cancel"];
	[cancel_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithFalse)]];
	[cancel_button setUsize: 100 height: 30];

	buttons = (GTKHBox*) [self actionArea];
	[buttons packStart:ok_button expand:NO fill:NO padding:0];
	[buttons packStart:cancel_button expand:NO fill:NO padding:0];

	// create the contents
	for( i = 0; i < 5; i++)
		h_boxes[i] = [GTKHBox hBoxWithHomogeneous:YES spacing:1];
	label = [GTKLabel labelWithStr:@"Printer name:\n(nothing for default)"];
	printerName = [GTKEntry entry];
	[h_boxes[0] packStart:label       expand:YES fill:YES padding:0];
	[h_boxes[0] packStart:printerName expand:YES fill:YES padding:0];
	radio_color = [GTKRadioButton radioButtonWithLabel:NULL 
				label:@"color"];
	radio_mono =  [GTKRadioButton radioButtonWithLabel: [radio_color group] 
				label:@"monochrome"];
	radio_solid = [GTKRadioButton radioButtonWithLabel: NULL 
				label:@"solid lines"];
	radio_dashed = [GTKRadioButton radioButtonWithLabel: [radio_solid group] 
				label:@"dashed lines"];
	label = [GTKLabel labelWithStr:@"\nColor style:"];
	[h_boxes[1] packStart:label       expand:YES fill:YES padding:0];
	[h_boxes[1] packStart:radio_color expand:YES fill:YES padding:0];
	label = [GTKLabel labelWithStr:@""];
	[h_boxes[2] packStart:label      expand:YES fill:YES padding:0];
	[h_boxes[2] packStart:radio_mono expand:YES fill:YES padding:0];
	label = [GTKLabel labelWithStr:@"\nLine style:"];
	[h_boxes[3] packStart:label       expand:YES fill:YES padding:0];
	[h_boxes[3] packStart:radio_solid expand:YES fill:YES padding:0];
	label = [GTKLabel labelWithStr:@""];
	[h_boxes[4] packStart:label        expand:YES fill:YES padding:0];
	[h_boxes[4] packStart:radio_dashed expand:YES fill:YES padding:0];

	v_box = (GTKVBox *) [self vbox];
	[v_box packStart:h_boxes[0] expand:YES fill:YES padding:0];
	[v_box packStart:[GTKHSeparator hSeparator] expand:YES fill:YES padding:0];
	[v_box packStart:h_boxes[1] expand:YES fill:YES padding:0];
	[v_box packStart:h_boxes[2] expand:YES fill:YES padding:0];
	[v_box packStart:[GTKHSeparator hSeparator] expand:YES fill:YES padding:0];
	[v_box packStart:h_boxes[3] expand:YES fill:YES padding:0];
	[v_box packStart:h_boxes[4] expand:YES fill:YES padding:0];

	[v_box setBorderWidth: 10];
	[radio_color setActive: YES];
	[radio_solid setActive: YES];

	[self setTitle: @"StatistX: Print gnuplot Plot"];
	[self setPosition: GTK_WIN_POS_CENTER ];

	[ok_button grabFocus];
	[self show];
	return self;
}
-(void)stopWithTrue {
	color_style = [radio_color getActive] ? sx_color : sx_mono;
	line_style  = [radio_solid getActive] ? sx_solid : sx_dashed;
	[GTKApp stopModalWithCode: YES];
}
-(void)stopWithFalse {
	[GTKApp stopModalWithCode: NO];
}
// returns the string contained in the printer text field
-(NSString*)getPrinterName {
	return [printerName getText];
}
// returns sx_solid or sx_dashed
-(int)getLineStyle {
	return line_style;
}
// returns sx_color or sx_mono 
-(int)getColorStyle {
	return color_style;
}

-(GTKButton*)okButton  {
	return ok_button;
}
-(GTKButton*)cancelButton {
	return cancel_button;
}

-(void) dealloc {
	[super dealloc];
}
@end
