/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 2000
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/* HTMLView.h
 * Very primitive viewer for HTML code. Very few tags supported!
 *
 */
#ifndef Class_HTMLView
#define Class_HTMLView

#include <Foundation/NSObject.h>
#include <Foundation/NSString.h>
#include <Foundation/NSCharacterSet.h>
#include <GToolKit/GToolKit.h>

// choose what you like
//#define HTML_FONT_FAMILY "times"
#define HTML_FONT_FAMILY "helvetica"

@interface Font: NSObject {
	NSString	* provider;
	NSString	* family;
	BOOL		bold, italic;
	unsigned	size;
	char		italic_indicator; // this is 'i' or 'o'
@private
	NSString	*string;
}
-initWithProvider: (NSString*) prov
		family: (NSString*) family 
		bold: (BOOL) is_bold
		italic: (BOOL) is_italic
		size: (unsigned) size;
-(void) setSize: (unsigned) size;
-(void) setBold: (BOOL) bold;
-(void) setItalic: (BOOL) italic;
/** This sets the character used to indicate the italic font ('i' or 'o'). */
-(void) setItalicID: (char) it_id;
-(GdkFont*)gdkFont;
-(NSString*)fontAsString;
@end // class Font

@interface HTMLView: GTKText {
	// This is YES if parser between '<' and '>'
	BOOL		in_tag;
	// fonts:
	Font		*f_h1, *f_h2, *f_h3, *f_h4, *f_text, *f_space;
	// font 'currently' used:s
	Font		*current_font;
	NSMutableString	*current_text;
	NSMutableString	*current_tag;
	BOOL		keep_nl, 	// preserve new lines
			html_file,	// file appears to be an HTML file
			in_head,	// true between <HEAD> and </HEAD>
			in_body;	// true between <BODY> and </BODY>
}
/** Display newlines as they are in the original HTML code. 
    After calling '-clear' this is always reset to its default
    value: 'YES' */
-(void)preserveNewlines: (BOOL) keep_nls;
/** Appends text, using the current text style definition. */
-(void) appendText: (NSString*) text;
-(void) appendHTMLCode: (NSString*) code;
-(void)evaluateTag: (NSString*) tag;
/** Clear the whole view. */
-(void) clear;
// Returns the number of displayed characters.
-(unsigned)length;
@end

#endif // Class_HTMLView
