/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 2000
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/* HTMLView.m
 * Very primitive viewer for HTML code. Very few tags supported!
 *
 */
#include "HTMLView.h"

#include <stdio.h>
#include <ctype.h>

#include <Foundation/NSArray.h>
#include <Foundation/NSObject.h>
#include <Foundation/NSString.h>
#include <Foundation/NSCharacterSet.h>
#include <GToolKit/GToolKit.h>

#ifndef SX_FONT_PROVIDER
#define SX_FONT_PROVIDER "*"
#endif

@implementation Font
-initWithProvider: (NSString*) _prov
		family: (NSString*) _family 
		bold: (BOOL) _bold
		italic: (BOOL) _italic
		size: (unsigned) _size {
	if(! [super init] ) return NULL;
	provider = [_prov retain];
	family = [_family retain];
	size = _size;
	bold = _bold;
	italic = _italic;
	italic_indicator = 'i';
	return self;
}
-init {
	return [self initWithProvider: @"adobe"
		family: (NSString*) @"times" 
		bold: (BOOL) NO
		italic: (BOOL) NO
		size: (unsigned) 120 ];
}
-(void) setSize: (unsigned) _size {
	size = _size;
}
-(void) setBold: (BOOL) _bold {
	bold = _bold;
}
-(void) setItalic: (BOOL) _italic {
	italic = _italic;
}
/** This sets the character used to indicate the italic font ('i' or 'o'). */
-(void) setItalicID: (char) it_id {
	italic_indicator = it_id;
}
-(GdkFont*)gdkFont {
	return gdk_font_load( [[self fontAsString] cString] );
}
-(NSString*)fontAsString {
	char	i, *b;
	b = bold ? "bold" : "medium";
	i = italic ?  italic_indicator : 'r';
	[string release];
	string = [NSString stringWithFormat:@"-%@-%@-%s-%c-*-*-*-%d-*-*-*-*-*-*",
			provider, family, b, i, size];
	return [string retain];
}
-(void) dealloc {
	[string release];
	[provider release];
	[family release];
	[super dealloc];
}
@end // class Font

@implementation HTMLView
- initWithHadj:(GTKAdjustment *) hadj vadj:(GTKAdjustment *) vadj {
	if(! [super initWithHadj: hadj vadj: vadj]) return NULL;
	[self setWordWrap: TRUE];
	// init fonts:
	f_h1 = [[Font alloc] initWithProvider: @SX_FONT_PROVIDER
			family: @HTML_FONT_FAMILY
			  bold: YES
			italic: NO
			  size: 180];
	f_h2 = [[Font alloc] initWithProvider: @SX_FONT_PROVIDER
			family: @HTML_FONT_FAMILY
			  bold: YES
			italic: NO
			  size: 170];
	f_h3 = [[Font alloc] initWithProvider: @SX_FONT_PROVIDER
			family: @HTML_FONT_FAMILY
			  bold: YES
			italic: NO
			  size: 140];
	f_h4 = [[Font alloc] initWithProvider: @SX_FONT_PROVIDER
			family: @HTML_FONT_FAMILY
			  bold: YES
			italic: NO
			  size: 120];
	f_text = [[Font alloc] initWithProvider: @SX_FONT_PROVIDER
			family: @HTML_FONT_FAMILY
			  bold: NO
			italic: NO
			  size: 120];
	f_space = [[Font alloc] initWithProvider: @SX_FONT_PROVIDER
			family: @HTML_FONT_FAMILY
			  bold: NO
			italic: NO
			  size: 40];
	current_font = f_text;
	html_file = NO;	// file *appears* to be an HTML file
	in_head = NO;	// true between <HEAD> and </HEAD>
	in_body = NO;	// true between <BODY> and </BODY>
	keep_nl = NO;
	return self;
}
/** Display new lines as they are in the original HTML code. */
-(void)preserveNewlines: (BOOL) keep_nls {
	keep_nl = keep_nls;
}
/** Appends text, using the current text style definition. */
-(void) appendText: (NSString*) text {
	unsigned length = [text length];
	if( !length ) return;
	if( !html_file ) keep_nl = YES; // this is probably no HTML code
	[self insert: [current_font gdkFont] fore:NULL 
		back:NULL chars: text length: length ];
}
-(void) appendHTMLCode: (NSString*) code {
	NSRange pos;
	NSArray	* rows = [code componentsSeparatedByString: @"\n"];
	NSString	* row;
	NSString	* a_string;
	char		a_char;
	unsigned	n_rows = [rows count], i, length;
	if( !current_text ) current_text = [NSMutableString new];

	for( i = 0; i < n_rows; i++ ) {
	   row = [NSMutableString stringWithString: [rows objectAtIndex: i]];
	   while( [row length] ) {
		if( in_tag ) { // between '<' and '>'
			pos = [row rangeOfString:@">"];
			if( pos.length ) {
				[current_tag appendString: [row substringToIndex: pos.location]];
				[self evaluateTag: current_tag];
				row = [row substringFromIndex:pos.location+pos.length];
				in_tag = NO;
			} else {
				[current_tag appendString: row];
				row = NULL;
			}
		} else { // normal text
			pos = [row rangeOfString:@"<"];
			if( pos.length ) {
				if( !in_head )
				   [self appendText: [row substringToIndex: pos.location]];
				row = [row substringFromIndex:pos.location+pos.length];
				in_tag = YES;
				current_tag = [NSMutableString new];
			} else {
				if( !in_head )
				   [self appendText: row];
				row = NULL;
			}
		}
	   }	// while length
	   length = [self getLength];
	   if( keep_nl ) [self appendText: @"\n"];
	   else if( !in_tag && !in_head && length ) {
	   	// get last character in the text field
		a_string = [self getChars: length-1 endPos:-1];
		a_char   = [a_string characterAtIndex: 0];
		if( !isspace( a_char ) )
		// if no white space, add new line
			[self appendText: @" "];
	   } // else
	}  // for each row
}
#define SX_comp( s1, s2 ) \
([s1 caseInsensitiveCompare: s2] == NSOrderedSame)

-(void)evaluateTag: (NSString*) _tag {
//fprintf(stderr, "%s\n", [_tag cString]);
	if( SX_comp(_tag, @"HTML") ) {
		keep_nl = NO;
		html_file = YES;
		return;
	}
	if( SX_comp(_tag, @"HEAD") ) {
		in_head = YES;
		in_body = NO;
		keep_nl = NO;
		html_file = YES;
		return;
	}
	if( SX_comp(_tag, @"BODY") ) {
		in_head = NO;
		in_body = YES;
		keep_nl = NO;
		html_file = YES;
		return;
	}
	if( SX_comp(_tag, @"/HEAD") ) {
		in_head = NO;
		return;
	}
	if( SX_comp(_tag, @"/BODY") ) {
		in_body = NO;
		return;
	}
	if( SX_comp(_tag, @"P") ) {
		[self appendText: @"\n"];
		[self appendText: @"\n"];
		return;
	}
	if( SX_comp(_tag, @"BR") ) {
		[self appendText: @"\n"];
		return;
	}
	if( SX_comp(_tag, @"B") ) {
		[current_font setBold: YES];
		return;
	}
	if( SX_comp(_tag, @"/B") ) {
		[current_font setBold: NO];
		return;
	}
	if( SX_comp(_tag, @"I") ) {
		[current_font setItalic: YES];
		return;
	}
	if( SX_comp(_tag, @"/I") ) {
		[current_font setItalic: NO];
		return;
	}
	if( SX_comp(_tag, @"H1") ) {
		if( [self getLength] ) [self appendText: @"\n"];
		current_font = f_space;
		[self appendText: @"\n"];
		current_font = f_h1;
		return;
	}
	if( SX_comp(_tag, @"H2") ) {
		if( [self getLength] ) [self appendText: @"\n"];
		current_font = f_space;
		[self appendText: @"\n"];
		current_font = f_h2;
		return;
	}
	if( SX_comp(_tag, @"H3") ) {
		if( [self getLength] ) [self appendText: @"\n"];
		current_font = f_space;
		[self appendText: @"\n"];
		current_font = f_h3;
		return;
	}
	if( SX_comp(_tag, @"H4") ) {
		if( [self getLength] ) [self appendText: @"\n"];
		current_font = f_space;
		[self appendText: @"\n"];
		current_font = f_h4;
		return;
	}
	if( SX_comp(_tag, @"/H1") 
	||  SX_comp(_tag, @"/H2") ) {
		current_font = f_space;
		[self appendText: @"\n"];
		[self appendText: @"\n"];
		current_font = f_text;
		return;
	}
	if( SX_comp(_tag, @"/H3") 
	||  SX_comp(_tag, @"/H4") ) {
		[self appendText: @"\n"];
		current_font = f_text;
		return;
	}
}
#undef SX_comp()

/** Clear the whole view. */
-(void) clear {
	[self deleteText:0 endPos:-1]; // clear text 
	html_file = NO;
	in_head = NO;
	in_body = NO;
	keep_nl = NO; // reset
	current_font = f_text;
	[f_text setBold: NO];
	[f_text setItalic: NO];
	[f_text setSize: 120];
}
// Returns the number of displayed characters.
-(unsigned)length {
	return [self getLength];
}
@end // class HTMLView
