/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	HistogramDialog.h - Dialog, that requests parameters 
 *	for histogram plot.
 *
 *	If not '+runHistogramDialogWithColumn:' is used,
 *	instances have to be run explicetly by calling
 
 	[GTKApp runModalForWindow: my_HistogramDialog];

 *
 *	Andreas Beyer, 1999
 */
#include "HistogramDialog.h"
#include "Column.h"
#include "Message.h"
#include "statist.h" // contains 'MCLASS'

#include <Foundation/NSObject.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSException.h>
#include <Foundation/NSString.h>

#include <GToolKit/GToolKit.h>

#include "math.h"

@implementation HistogramDialog
+(BOOL)runDialogWithColumn: (Column*) _col {
	BOOL result;
	HistogramDialog* dialog =
		[[HistogramDialog alloc] initWithColumn: _col];
	if( ! dialog ) 	[NSException raise: NSGenericException
		format: @"+runDialogWithColumn: Can't create HistogramDialog"];
	result = [GTKApp runModalForWindow: dialog];
	[dialog destroy];
	[dialog release];
	return result;
}
+dialogWithColumn: (Column*) _col {
	return [ (HistogramDialog*) [[self alloc] autorelease]
		initWithColumn: _col];
}

-initWithColumn: (Column*) _col {
	GTKVBox *v_box;
	GTKHBox *buttons;
	GTKTable* contents
		= [GTKTable tableWithRows: 3 columns: 3 homogeneous: YES ];
	GTKLabel* label;
	GTKAdjustment* adjustment;
	GTKTooltips *tips = [GTKTooltips tooltips];
	double min_v, max_v, min_min, min_max, max_min, max_max;
	double stepInc;
	[super init];
	col = [_col retain];
	// make the two buttons
	ok_button = [GTKButton buttonWithLabel: @"OK"];
	[ok_button setUsize: 100 height: 30];
	[ok_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithTrue)]];
	cancel_button = [GTKButton buttonWithLabel: @"Cancel"];
	[cancel_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithFalse)]];
	[cancel_button setUsize: 100 height: 30];

	buttons = (GTKHBox*) [self actionArea];
	[buttons packStart:ok_button expand:NO fill:NO padding:0];
	[buttons packStart:cancel_button expand:NO fill:NO padding:0];

	// create the contents
	label = [GTKLabel labelWithStr: @"number of classes"];
	[label setAlignment: 1 yalign: 0];
	[contents attach: label
		leftAttach: 0
		rightAttach: 1 
		topAttach: 0
		bottomAttach: 1
		xoptions: GTK_EXPAND | GTK_FILL
		yoptions: GTK_EXPAND
		xpadding: 1
		ypadding: 1
	];
	adjustment = [GTKAdjustment adjustmentWithValue: 0
				lower: 0 
				upper: MCLASS // global variable, defined by statist
				stepIncrement:1
				pageIncrement:5 
				pageSize:0
			];
	n_classes = [GTKSpinButton spinButtonWithAdjustment:adjustment 
		climbRate:0 digits:0];
	[n_classes retain];
	[n_classes setNumeric: TRUE];
	[tips setTip:n_classes tipText:@"Set to '0' for auto classification" tipPrivate:nil];
	[contents attach: n_classes
		leftAttach: 1
		rightAttach: 3 
		topAttach: 0
		bottomAttach: 1
		xoptions: GTK_EXPAND | GTK_FILL
		yoptions: GTK_EXPAND
		xpadding: 1
		ypadding: 1
	];
	label = [GTKLabel labelWithStr: @"min"];
	[label setAlignment: 1 yalign: 0];
	[contents attach: label
		leftAttach: 0
		rightAttach: 1 
		topAttach: 1
		bottomAttach: 2
		xoptions: GTK_EXPAND | GTK_FILL
		yoptions: GTK_EXPAND
		xpadding: 1
		ypadding: 1
	];
	min_v = min_min = [col min];
	if( min_min < 0 ) min_min *= 1.5;
	else min_min = 0;
	max_v = min_max = [col max];
	stepInc = fabs(min_v/10);
	if( stepInc > 5 ) stepInc = (double) (int) stepInc; // round
	adjustment = [GTKAdjustment adjustmentWithValue: min_v
				lower: min_min 
				upper: min_max
				stepIncrement: stepInc
				pageIncrement: stepInc * 10
				pageSize:0
			];
	min = [GTKSpinButton spinButtonWithAdjustment:adjustment 
		climbRate:0 digits:3];
	[min retain];
	[min setNumeric: TRUE];
	[contents attach: min
		leftAttach: 1
		rightAttach: 3 
		topAttach: 1
		bottomAttach: 2
		xoptions: GTK_EXPAND | GTK_FILL
		yoptions: GTK_EXPAND
		xpadding: 1
		ypadding: 1
	];
	label = [GTKLabel labelWithStr: @"max"];
	[label setAlignment: 1 yalign: 0];
	[contents attach: label
		leftAttach: 0
		rightAttach: 1 
		topAttach: 2
		bottomAttach: 3
		xoptions: GTK_EXPAND | GTK_FILL
		yoptions: GTK_EXPAND
		xpadding: 1
		ypadding: 1
	];
	max_min = min_v;
	max_max = max_v;
	if( max_v < 0 ) max_max = 0;
	else max_max *= 1.5;
	stepInc = fabs((max_v)/10);
	if( stepInc > 5 ) stepInc = (double) ((int) stepInc); // round
	adjustment = [GTKAdjustment adjustmentWithValue: max_v
				lower: max_min 
				upper: max_max
				stepIncrement: stepInc
				pageIncrement: stepInc * 10
				pageSize:0
			];
	max = [GTKSpinButton spinButtonWithAdjustment:adjustment 
		climbRate:0 digits:3];
	[max retain];
	[max setNumeric: TRUE];
	[contents attach: max
		leftAttach: 1
		rightAttach: 3
		topAttach: 2
		bottomAttach: 3
		xoptions: GTK_EXPAND | GTK_FILL
		yoptions: GTK_EXPAND
		xpadding: 1
		ypadding: 1
	];

	v_box = (GTKVBox *) [self vbox];
	[v_box packStart:contents expand:YES fill:YES padding:0];
	[v_box setBorderWidth: 10];

	[self setTitle: [NSString stringWithFormat: 
		@"Histogram Parameters [%s]", [col getLabel]]];
	[self setPosition: GTK_WIN_POS_CENTER ];

	[self show];
	[ok_button grabFocus];
	return self;
}
-(void)stopWithTrue {
	double min_v, max_v;
	min_v = [self getMin]; max_v = [self getMax];
	if( min_v >= max_v ) {
		[Message messageWithContents: @"Maximum has to be larger\n\
		than minimum." title: @"Histogram: Error"];
		return;
	}
	[GTKApp stopModalWithCode: YES];
}
-(void)stopWithFalse {
	[GTKApp stopModalWithCode: NO];
}
-(int) getNClasses {
	return [n_classes getValueAsInt];
}
-(double) getMin {
	return [min getValueAsFloat];
}
-(double) getMax {
	return [max getValueAsFloat];
}
-(GTKButton*)okButton  {
	return ok_button;
}
-(GTKButton*)cancelButton {
	return cancel_button;
}

-(void) dealloc {
	[n_classes release];
	[min release];
	[max release];
	[super dealloc];
}
@end
