/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/* LineDialog.m
 * Requests a single line of text from the user.
 *
 * Andreas Beyer, 1999
 */
#include <Foundation/NSException.h> 
#include "LineDialog.h"
 
@implementation LineDialog
+(NSString*)runDialogWithTitle: (const NSString*) title 
			message: (NSString*) message 
			strings: (NSArray*)  list_of_strings {
	return [self runDialogWithTitle: title
			message: message 
			   text: (NSString*)[list_of_strings objectAtIndex: 0]];		
}

+(NSString*)runDialogWithTitle: (const NSString*) title
			message: (NSString*) message 
			   text: (NSString*) initial_text {
	LineDialog* dialog = [[LineDialog alloc] initWithTitle: title 
				message: message text: initial_text];
	if( ! dialog ) 	[NSException raise: NSGenericException
		format: @"+runDialogWithMessage: Can't create LineDialog: %s", title];
	if( [GTKApp runModalForWindow: dialog] ) {
		NSString* result = [dialog getText];
		[dialog destroy];
		[dialog release];
		return result;
	}
	[dialog destroy];
	[dialog release];
	return NULL;		
}

/** This creates a dialog with the default buttons 'OK' and 'Cancel' */
-initWithTitle: (const NSString*) title
		message: (NSString*) message 
		strings: (NSArray*) string_array {
	[self initWithTitle: title message: message 
			   text: (NSString*)[string_array objectAtIndex: 0]];		
	return self;
}

/** This creates a dialog with the default buttons 'OK' and 'Cancel' */
-initWithTitle: (const NSString*) title
		message: (NSString*) message 
		text:  (const NSString*) initial_text {
	GTKVBox *v_box;
	GTKHBox *buttons;
	GTKLabel* label;
//	NSString * text = [NSString stringWithCString: [initial_text cString]];
	NSString * text = [initial_text copy];
	[super init];
	// make the two buttons
	ok_button = [GTKButton buttonWithLabel: @"OK"];
	[ok_button setUsize: 100 height: 30];
	[ok_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithTrue)]];
	cancel_button = [GTKButton buttonWithLabel: @"Cancel"];
	[cancel_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithFalse)]];
	[cancel_button setUsize: 100 height: 30];

	buttons = (GTKHBox*) [self actionArea];
	[buttons packStart:ok_button expand:NO fill:NO padding:0];
	[buttons packStart:cancel_button expand:NO fill:NO padding:0];

	// create the contents
	entry = [GTKEntry new];
	[entry connectSignal:@"activate" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithTrue)]];
	
	if( [text length] ) {
		[entry setText: text];
		[entry selectRegion: 0 end: -1];
	}
	v_box = (GTKVBox *) [self vbox];
	if( message && [message length] > 0 ) {
		label = [GTKLabel labelWithStr: message];
		[label setPadding: 5 ypad: 10];
		[v_box packStart:label expand:YES fill:YES padding:0];
	}
	[v_box packStart:entry expand:YES fill:YES padding:0];
	[v_box setBorderWidth: 10];

	[self setTitle: title];
	[self setPosition: GTK_WIN_POS_CENTER ];

	[self show];
	[entry grabFocus];
	return self;
}
-(void)setText: (const NSString*) text {
}
-(void)stopWithTrue {
	[GTKApp stopModalWithCode: YES];
}
-(void)stopWithFalse {
	[GTKApp stopModalWithCode: NO];
}

-(GTKButton*)okButton  {
	return ok_button;
}
-(GTKButton*)cancelButton {
	return cancel_button;
}
-(NSString*) getText {
	return [entry stringValue];
}
-(void) dealloc {
	[entry release];
	[super dealloc];
}
@end
