/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/


#include <stdio.h>
#include <string.h>

#include <GToolKit/GToolKit.h>
#include <Foundation/NSAutoreleasePool.h>

#include "DataDisplay.h"
#include "sx_statist.h"

#define OPTIONS_STRING "Options:\n\
  --help, -?\t Print this message and exit successfully.\n\
  --version\t Print version information and exit successfully.\n\
  -\t\t Take data from standard input.\n\
Please refer to the X11 documentation for help to generic X-options.\n\n"

#define STATISTX_HOMEPAGE "<URL:http://www.usf.uni-osnabrueck.de/~abeyer/private/StatistX/>\n"

void print_help(const char* prog_name) {
	printf("usage: %s [file_name ...] [options] [generic X-options ...]\n", prog_name);
	printf(OPTIONS_STRING);
	printf(STATISTX_HOMEPAGE);
}

#undef OPTIONS_STRING
#undef STATISTX_HOMEPAGE

int parse_options(int argc, char *argv[]) {
/** Use this function ONLY for those command line options, which
    cause StatistX to stop operation. Any other options should
    be parsed in [Manager init] or [GTKApplication init].
*/
	int i;
	int help = 0, version = 0;
	/* GNU standard requires that --version causes the program
	   to do nothing but printing the version info. Thus, any 
	   appearance of --version should suppress any other output. 
	*/
	for(i = 1; i < argc; i++) {
	    if( !strcmp(argv[i], "--version") ) 
		version = 1;
	    if( !strcmp(argv[i], "--help") || !strcmp(argv[i], "-?"))
		help = 1;
	}
	if( version ) {
    		printf("StatistX version %s\npublished under GPL\n", [[Manager version] cString]);
#ifdef DEBUG
		printf( "statist version %s\n", [[Manager statistVersion] cString] );
#endif
		return 0;
	}
	if( help ) {
		print_help(argv[0]);
	 	return 0;
	}
	return 1; /* success, continue operation */
}

int main (int argc, char *argv[])
{
    int f;	// number of file on command line
    NSAutoreleasePool *pool = [NSAutoreleasePool new];
    Manager *ma;
#ifdef LIB_FOUNDATION_LIBRARY
    extern char **environ;
    [NSProcessInfo initializeWithArguments:argv count:argc environment:environ];
#endif

    if( argc > 1 ) {
	if( !parse_options(argc, argv) ) {
		[pool release];
		return 0;
	}
    }

   // defined in sx_statist.m Call this before Manager init.
   initialize_statist();

    // Exists only once, in 'GTKApp' ...
    ma = [[Manager alloc] initWithArgc:&argc argv:&argv];

   for( f = 1; f < argc; f++ ) {
   	if( !strcmp(argv[f], "-") )
		[ma readDataFromStdin];
	else
		[ma openFile: [NSString stringWithCString: argv[f]]];
   }

    [GTKApp run];
#ifdef DEBUG
	fprintf(stderr, "cleaning up ...\n");
#endif // DEBUG
    [pool release];
    return 0;
}
