/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	Manager.h - catch events and handle them.
 *
 *
 *	Andreas Beyer, 1998
 */
#ifndef Class_Manager
#define Class_Manager

// under linux signals defined in <asm/signal.h>
#include <signal.h>
#include <stdio.h>
#include <Foundation/NSObject.h>
#include <Foundation/NSDictionary.h>
#include <GToolKit/GToolKit.h>

#include "SXAction.h"
#include "TextPanel.h"
#include "sx_defs.h"

#ifdef IN_MANAGER_M
static int caught_signals[] = { SIGTERM, SIGHUP, SIGQUIT,
				-1 // terminator, very importand!
			};
#endif // IN_MANAGER_M

@class DataDisplay;
@class NSMutableArray;
@class FileList;
@class SXKernel;

@interface Manager : GTKApplication {
	GTKWindow 		*top_window;
	GTKItemFactory 	*item_factory;
    	GTKWidget		*menubar; // this must be type of GTKWidget!
	GTKToolbar		*toolbar;
 	GTKNotebook		*notebook;
	GTKStatusbar	*statusbar;
	NSString		*lastOpenDir;	// directory of last opened file
	SXKernel		*kernel;		// kernel doing the statistical work
	FileList		*file_list;	// list of recently opened files in menu
@private
	TextPanel	*debug_window;
	NSMutableArray * controlled_windows;
	BOOL terminated;
}
// return the version of StatistX
+(NSString*)version;
// return the version of statist
+(NSString*)statistVersion;
// return the generic Manager (which is the same object as GTKApp)
+(Manager*)getManager;
// will terminate the program:
-(void)gotSignal: (int) sig;

-initWithArgc:(int *) argc argv:(char ***) argv;
// set all signals in 'catched_signals':
-(void) setSignals;
// Called during init. Sets up the previously created toolbar.
-(void) setUpToolbar;
-(void) addToToolbarItem: (GTKMenuItem*) item label: (NSString*) label toolTip: (NSString*) name xpmIcon: (gchar**) icon;

/** This will open a log-file calles 'path'. A previously opened log-file
   will be closed. */
-(void) setLogFileName: (NSString*) path;
/** Return FILE pointer to log-file. Returns NULL if no log-file open. */
-(FILE*)getLogFile;

-(void) registerDataDisplay: (DataDisplay*) dd;
-(void) unregisterDataDisplay: (DataDisplay*) dd;
-(unsigned) countControlledDataDisplays;

// read data from stdin instead of reading from a file
-(void)readDataFromStdin;
-(void)openFile: (NSString *) file_name;
-(void)addDataDisplay: (DataDisplay*) dd;
// Register 'callback' for action 'menu_path' and create
// menu entry. Will always invoke the current DataSet.
-(void)registerAction: (NSString*) menu_path target: target selector: (SEL) callback;
// Will return NULL if no file open.
-(DataDisplay*)getCurrentDataDisplay;
// Will return the columns of the current DataDisplay, NULL if no file open.
-(NSArray*)getSelectedCols;
-(SXKernel*)getKernel;

-(void)menu_openFile: sender;
// close current file
-(void)menu_closeFile:sender;
-(void)menu_selectAll;
-(void)menu_displayStandard;
-(void) ok:sender;	// open panel
-(void) cancel:sender;	// open panel

-(void)printOnDebug:(NSString*) debug_msg;
/* This method just displays a message on the screen. It doesn't
   stop the program or do anything else. It thus doesn't replace exception
   handling!
 */
-(void)reportException: (NSException*) exception;
@end

#endif // Class_Manager
