/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	Message.h - Dialog, that displays a single string.
 *	(Such like an error message.)
 *
 *	Andreas Beyer, 1998
 */
#include "Message.h"

#include <Foundation/NSObject.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSException.h>
#include <Foundation/NSString.h>

#include <GToolKit/GToolKit.h>


static void clicked (GtkWidget *widget)
{
    [GTKApp stopModalWithCode:[GTK_GET_OBJECT(widget) tag]];
}

static gint delete (GtkWidget *widget, GdkEvent *event)
{
    clicked(widget);
    return YES;
}

static GTKButton *insert (GTKBox *box, NSString *label, int tag)
{
    GTKButton *button = [GTKButton buttonWithLabel:label];
    GtkButton *gtk_button = [button gtk];

    [button setTag:tag];
    [box packStart:button expand:YES fill:NO padding:0];
    gtk_misc_set_padding((GtkMisc *) gtk_button->child, 4, 0);
    GTK_WIDGET_SET_FLAGS(gtk_button, GTK_CAN_DEFAULT);
    gtk_signal_connect((GtkObject *) gtk_button, "clicked",
		       (GtkSignalFunc) clicked, NULL);
    return button;
}

/*
 * Display a modal message box or alert panel.
 * Return value is n-1 if button number n is clicked (e.g. 0 for button1).
 * Same as GTKRunMessageBox() in GTKApplication.m but places the panel in the
 * centre of the screen.
 */
int SXRunMessageBox (NSString *title, NSString *format, NSString *button1,
		      NSString *button2, NSString *button3, ...)
{
    GTKDialog *window = [GTKDialog dialog];
    GTKBox *hbox = (GTKBox *) [window actionArea];
    GTKBox *vbox = (GTKBox *) [window vbox];
    GTKButton *focus = nil;
    GTKLabel *label;
    NSString *text;
    int result, tag = 0;
    va_list ap;

    if (button1) focus = insert(hbox, button1, tag);
    if (button2) insert(hbox, button2, ++tag);
    if (button3) insert(hbox, button3, ++tag);
    if (focus) [focus grabDefault], [focus grabFocus];
    if (title) [window setTitle:title];

    va_start(ap, button3);
    text = [[NSString alloc] initWithFormat:format arguments:ap];
    label = [GTKLabel labelWithStr:text];
    [text release];
    va_end(ap);

    [label setPadding:10 ypad:10];
    [label setJustify:GTK_JUSTIFY_LEFT];
    [vbox packStartDefaults:label];
    [window setTag:tag];
    [window setPosition: GTK_WIN_POS_CENTER ];

    gtk_signal_connect([window gtk], "delete_event", (GtkSignalFunc) delete, 0);
    result = [GTKApp runModalForWindow:window];
    [window destroy];
    return result;
}

@implementation Message
+messageWithContents: (const NSString*) message title: (const NSString*) title {
	return [ (Message*) [[self alloc] autorelease]
		initWithContents: message title: title];

}
-initWithContents: (const NSString*) message title: (const NSString*) title  {
	GTKLabel* label = [GTKLabel labelWithStr: message];
	[label setPadding: 5 ypad: 10];
	return [super initWithContents: label title: title];
}
-(const NSString*) message {
	return contents;
}

@end
