/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/** SXFileHandle.h
 *
 *	This class exists because the NSFileHandle implementation
 *	is mostly wrong.
 *	Additionally it provides line-wise reading.
 *	This class IS NOT unicode safe!
 *
 *	Andreas Beyer, 1999
 */
#ifndef Class_SXFileHandle
#define Class_SXFileHandle

#include <Foundation/NSObject.h>
#include <stdio.h>

@class NSString;
@class NSMutableString;
@class NSFileHandle;

@interface SXFileHandle: NSObject {
	FILE*		stream;
@private
	/* This string variable is usually == nil.
	   It's only != nil if the current line has more than
	   SX_BUF_LEN -1 chars.
	 */
	NSMutableString*	currentLine;
	BOOL			fileIsClosed;
	unsigned		next_line_no;
	unsigned		total_line_no; // total number of lines in file
}
/* These initializers create objects which will close the resource 
   on deallocation.
 */
+fileHandleWithCFilePointer: (FILE*) fp;
+fileHandleWithPath: (NSString*) path;
// designated initializer:
-initWithCFilePointer: (FILE*) fp;
-initWithPath: (NSString*) path;
/* Returns a newly created string containing the next line of a resource (file, port, etc.)
  nil if no data available.
  The calling procedure/object is responsible for releasing the new string.
 */
-(NSString*) getNextLine;
/* Return total number of lines.
   This method returns 0 if the FileHandle was not created using initWithPath:
   In the case of very long lines, the return value may be larger than the
   actual number of lines. totalNumberOfLines is just an approximation in
   that case!
 */
-(unsigned)totalNumberOfLines;
/* Return number of NEXT line. -> previous line == numberOfNextLine-1
   Start counting with 1 (first line).
   Does not indicate that the next line exists!
 */
-(unsigned) numberOfNextLine;
-(void)close;
@end
#endif // Class_SXFileHandle
