/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/** SXFileHandle.m
 *
 *	Andreas Beyer, 1999
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h> // calloc()

#include <Foundation/NSString.h>
#include <Foundation/NSData.h>
#include <Foundation/NSException.h>
#include <Foundation/NSCharacterSet.h>

#include "SXFileHandle.h"
#include "sx_defs.h"

static NSStringEncoding encoding;
static NSCharacterSet*	line_endings;

@implementation SXFileHandle
+(void) initialize {
	unichar	u_line_endings[2];
	NSString *	t_str;
	encoding = [NSString defaultCStringEncoding];
	u_line_endings[0] = '\n'; 
	u_line_endings[1] = '\r'; 
	t_str = @"\n\r";
	line_endings = [NSCharacterSet characterSetWithCharactersInString: t_str];
	if( !line_endings ) [NSException raise: NSMallocException
				format: @"SXFileHandle+initialize: Not enough memory."];
	
}

+fileHandleWithCFilePointer: (FILE*) fp {
	return [[[self alloc] initWithCFilePointer: fp] autorelease];
}
+fileHandleWithPath: (NSString*) path {
	return [[[self alloc] initWithPath: path] autorelease];
}
-initWithPath: (NSString*) path {
/* Since this FileHandle represents an actual file and no
   network connection, we can determine the total number
   of lines.
 */
 	char buf[1024];
	FILE* fp = fopen( [path cString], "r" );
	if( !fp ) {
		[self release];
		return NULL;
	}
	// This is an *approximation* of the actual number of lines.
	while( fgets( buf, 1024, fp ) ) total_line_no++;
	rewind(fp);
	return [self initWithCFilePointer: fp];
}
// designated initializer
-initWithCFilePointer: (FILE*) fp {
	[super init];
	next_line_no = 1;
	stream = fp;
	if( stream )fileIsClosed = NO;
	else		fileIsClosed = YES;
	return self;	
}

// returns a new string containing the next line of a resource (file)
// nil if no data available
-(NSString*) getNextLine {
	char		buf[SX_BUF_LEN],
			*cString, // the line as C string
			*l_del;   // pos of line delimiter in buf
	unsigned	str_len;
	NSString*	line;

	if( fileIsClosed ) return NULL;
	if( !(cString = fgets( buf, SX_BUF_LEN, stream )) ) {
	/* EOF reached. */
		[self close];
		return NULL;
	}
	l_del = strchr(cString, '\n');
	if( !l_del && strlen( cString ) < SX_BUF_LEN-1 ) {
	// Is a binary file.
		[self close];
		return NULL;
	}
	if( l_del ) { // remove \n
		 *l_del = '\0';
		// remove possible \r
		if( strlen( cString ) && *(l_del-1) == '\r' )
			*(l_del-1) = '\0';
	}
	line = [[NSString alloc] initWithCString:cString];
	if( !line ) [NSException raise: NSMallocException
				format: @"-getNextLine: Not enough memory."];
	if( !l_del ) { // line is longer than SX_BUF_LEN-1 characters ...
		if( ! currentLine 
		&&  !(currentLine = [[NSMutableString alloc] 
				initWithCapacity: SX_BUF_LEN])
		) [NSException raise: NSMallocException
				format: @"-getNextLine: Not enough memory."];
		[currentLine appendString: line];
		return [self getNextLine];
	}
	if( currentLine ) {
		[currentLine appendString: line];
		[line release];
		line = [[NSString alloc] initWithString: currentLine];
		if( !line ) [NSException raise: NSMallocException
				format: @"-getNextLine: Not enough memory."];
		[currentLine release];
		currentLine = NULL;
	}
	next_line_no++;
//fprintf(stderr, "line %d: >%s<\n", [self numberOfNextLine]-1, [line cString]);
	return line;
}
-(unsigned)totalNumberOfLines {
	return total_line_no;
}
-(unsigned) numberOfNextLine {
// start counting with 1 (first line)
	return next_line_no;
}
-(void)close {
	if( fileIsClosed ) return;
	fclose(stream);
	fileIsClosed = YES;
}
-(void) dealloc {
	[self close];
	[super dealloc];
}
@end
