/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999 - 2000
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	SXKernel.h - SXKernel provides axcess to 'statist' functions.
 *
 *	Andreas Beyer, 1999
 */
#ifndef Class_SXKernel
#define Class_SXKernel

#include <stdlib.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSObject.h>

#include "Manager.h"
@class Column;

@interface SXKernel : NSObject {
	Manager *manager;
@private
	// The kerenl changes LC_NUMERIC to the default C locale
	// before calling statist.
	char * SX_old_locale, * SX_saved_locale;
}
+kernelWithManager: (Manager*) m;
-initWithManager: (Manager*) m ;
-(void)registerActionsAtManager;

/*  Get exactly number_of_cols
 *  columns from the manager.
 *  Creates error message otherwise.
 */
-(NSArray*)getCols: (unsigned) number_of_cols;
// get at least one column from the manager, error, if no columns
-(NSArray*)getNCols;

//==========================================================================
//>>>>>> statistical stuff:

//==================== Standard Menu =======================================
-(void) histogramAuto: sender;
-(void) histogramUserDefined: sender;
-(void) histogramForCol: (Column*) col 
		  n_classes: (int) n // if zero -> auto determination of classes
		  	  max: (double) max
			  min: (double) min;
-(void) probitAnalysis: sender;
-(void) probitAnalysisDose: (Column*) c_1 
			   count: (Column*) c_2 
			  effect: (Column*) c_3;
-(void) probitAnalysisNoCountDose: sender;
-(void) probitAnalysisDose: (Column*) c_1 effect: (Column*) c_2;
-(void) outlier: sender;
-(void) outlierOfCol: (Column*) col;
//==================== Regression & Correlation Menu =======================

-(void) rankOrder: sender;
-(void) rankOrderForCols: (Column*) c1 : (Column*) c2;
-(void) linearRegression: sender;
-(void) linearRegressionForCols: (Column*) c1 : (Column*) col_2;
-(void) multipleRegression: sender;
-(void) multipleRegressionTreatment: (Column*) treat dependent: (NSArray*) cols;
-(void) partLinearCorr: sender;
-(void) partLinearCorrForCols: (NSArray*) cols;
-(void) polynomialRegresssion: sender;
-(void) polynomialRegresssionWithOrder: (unsigned) order 
				: (Column*) col_1 : (Column*) col_2;
-(void) correlationMatrix: sender;
-(void) corrMatrixForCols: (NSArray*) cols;
// Spearmann rank-correlation
-(void) rankMatrix: sender;
-(void) rankMatrixForCols: (NSArray*) cols;
-(void) crossValidate: sender;
-(void) crossValidateIndependent: (Column*) indep dependent: (NSArray*) cols;
-(void) randomTupel: sender;
-(void) randomTupelWithNTupels: (unsigned) number_of_tupels
			 independent: (Column*) indep 
			   dependent: (NSArray*) cols;
// First column must contain only 1's or 0's.
-(void) pointBiserialRegression: sender;
-(void) pointBiserialRegForCols: (Column*) col_1 : (Column*) col_2;
//==================== Tests Menu ===========================================
// t-Test zum Vergleich zweier Mittelwerte aus Stichproben
-(void) tTest: sender;
-(void) tTestForCols: (Column*) c1 : (Column*) c2;
// t-test zum Vergleich paarweise erhobener Stichproben
-(void) pairTTest: sender;
-(void) pairTTestForCols: (Column*) c1 : (Column*) c2;
// Test auf Normalverteilung (KS-Lillifors-Test)
-(void) ksTest: sender;
-(void) ksTestForCol: (Column*) col;
// Chi^2-Vierfeldertafel
-(void) fourFieldsTest: sender;
-(void) fourFieldsTestForCols: (Column*) c1 : (Column*) c2;
// CHI^2 Zwei-Merkmale-Tafeltest
-(void) twoItemsTest: sender;
-(void) twoItemsTestForCols: (NSArray*) cols;
// u-Test (Test auf Unabhaengigkeit 2er Stichproben)
-(void) uTest: sender;
-(void) uTestForCols: (Column*) c1 : (Column*) c2;
// H-Test (Kruskal-Wallis) fuer k unabhaengige Stichproben
// (MIND. 3 SPALTEN)
-(void) hTest: sender;
-(void) hTestForCols: (NSArray*) cols;
// Wilcoxon-Rang-Test fuer paarweise erhobene Stichproben
-(void) wilcoxonTest: sender;
-(void) wilcoxonTestForCols: (Column*) c1 : (Column*) c2;
// Chi^2-Test auf gleiche Haeufigkeiten
-(void) equalFrequTest: sender;
-(void) equalFrequTestForCol: (Column*) col;
// Chi^2-Test auf Gleicheit von gemessenen mit theor. Haeufigkeiten
// 1st column: measured distribution
// 2nd column: expected distribution
-(void) compareFrequTest: sender;
-(void) compareFrequTestMeasured: (Column*) m expected: (Column*) e;

//============================= END STATISTCS ==============================
//==========================================================================

@end
#endif // Class_SXKernel
