/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/** SXProgressBar.h
 *
 *	Convinient access to a progress bar.
 *	Only one progress bar allowed at one time!
 *
 *	Andreas Beyer, 1999
 */
#ifndef Class_SXProgressBar
#define Class_SXProgressBar

#include <Foundation/NSObject.h>
#include <GToolKit/GToolKit.h>

@class NSString;

@protocol ProgressDelegate
/* Returns the current state in percent (between 0 & 1) or -1.
   If a value less than 0 is returned, the dialog will be destroyed
   and the timeout handler will be stoped.
   -stopOperation will also be called in this case.
 */
-(double) state;
-(void)stopOperation;	// Called when the user selects "Cancel"
@end

// in msec.
#define SXTimeOutLength 200

@interface SXProgressBar: GTKDialog {
	id <ProgressDelegate>	p_delegate;
	GTKProgressBar*	pbar;
	GTKLabel*		label;
	gint 			timeout_tag;
}
+(SXProgressBar*)objectForPointer:(void*)pnt;
+progressBarWithDelegate:delegate title: (NSString*) title;
-initWithDelegate:delegate title: (NSString*) title;
// Set a user info, which is displayed above the progress bar.
-(void) setInfo: (NSString*) info;
-(int)updateState;
@end
#endif // Class_SXProgressBar
