/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/** SXProgressBar.m
 *
 *	Convinient access to a progress bar.
 *	Only one progress bar allowed at one time!
 *
 *	Andreas Beyer, 1999
 */
#include "SXProgressBar.h"
#include <Foundation/NSString.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSException.h>

static NSMutableArray* allObjects;
static gint timeout_callback( gpointer data ) {
	gint res = [(id)data updateState];
	return res;
}

@implementation SXProgressBar
+(void)initialize {
	allObjects = [[NSMutableArray alloc] initWithCapacity: 2];
	if(!allObjects)
		[NSException raise: NSMallocException
		format: @"+initialize: Can't create array for objects."];
}
+(SXProgressBar*)objectForPointer:(void*)pnt {
	unsigned index = [allObjects indexOfObject: pnt];
	if( index == NSNotFound ) return NULL;
	return [allObjects objectAtIndex: index]; 
}

+progressBarWithDelegate: _delegate title: (NSString*) title {
	return [ (SXProgressBar*) [[self alloc] autorelease]
		initWithDelegate:_delegate title: title];
}
-initWithDelegate:_delegate title: (NSString*) title {
	GTKButton* cancel = [GTKButton buttonWithLabel: @"Cancel"];
	GTKVBox*   v_box; // do not initialize before init
	GTKHBox*   h_box; // do not initialize before init
	GtkButton *gtk_button = [cancel gtk];

	[super init];
	[allObjects addObject: self];

	if( ! [_delegate conformsToProtocol: @protocol(ProgressDelegate)] )
		[NSException raise: NSInvalidArgumentException
		format: @"-init: Delegate does not conform to ProgressDelegate protocol."];
	p_delegate = (id <ProgressDelegate>) _delegate;
	
	v_box = (GTKVBox*) [self vbox];
	h_box = (GTKHBox*) [self actionArea];
	
	[cancel setUsize: 85 height: 35];
      GTK_WIDGET_SET_FLAGS(gtk_button, GTK_CAN_DEFAULT);
	[cancel connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(destroy)]];
	[h_box packStart:cancel expand:YES fill:NO padding:0];

	label = [[GTKLabel labelWithStr:@"work in progress ..."] retain];
	pbar  = [[GTKProgressBar progressBar] retain];
	[v_box packStart:label expand:YES fill:YES padding:0];
	[v_box packStart:pbar  expand:YES fill:YES padding:0];
	[v_box setBorderWidth: 10];

	[self setTitle: title];
	[self setPosition: GTK_WIN_POS_CENTER ];

	[cancel grabDefault];
	[cancel grabFocus];
	[self show];
	timeout_tag = gtk_timeout_add(SXTimeOutLength, &timeout_callback, self);
	// force drawing
	while (gtk_events_pending()) gtk_main_iteration();
	return self;
}
// Set a user info, which is displayed above the progress bar.
-(void) setInfo: (NSString*) info {
	[label setText: info];
}
-(int) updateState {
	double state = [p_delegate state];
	if( state < 0 ) { // stop progress bar
		[self destroy];
		return 0;
	}
	if( state <= 1 && state >= 0) [pbar update: state];
	// force drawing
	while (gtk_events_pending()) gtk_main_iteration();
	return 1;
}

-(void)destroy {
	[p_delegate stopOperation];
	gtk_timeout_remove( timeout_tag );
	[super destroy];
}
-(void) dealloc {
	[allObjects removeObject: self];
	[pbar release];
	[label release];
	[super dealloc];
}

@end // implementation
