/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/* SXSettings.h
 * SXSettings manages general settings of the publications
 * like pathnames, etc.
 */
#ifndef Class_SXSettings
#define Class_SXSettings

#include <Foundation/NSObject.h>
#include <Foundation/NSString.h>
#include <GToolKit/GToolKit.h>

// If one parameter is  of type SXVTpath it gets a BrowseButton in the dialog.
enum SXValueType {
	SXVTstring,
	SXVTpath
};

typedef struct {
	NSString *	key;
	NSString *	value;
	NSString *	description;
	// determines the behavior of the dialog
	int		value_type;
} SXSettings_Desc;

static SXSettings_Desc DefaultSettings[] = {
   { @"DATA_DIR", @".", @"Data directory", SXVTpath},
   { @"PRINT_CMD", @"lpr", @"Print command", SXVTstring},
   { @"LOG_FILE", @"", @"Name of log-file", SXVTpath},
   { @"HELP_DIR", @SX_DOC_DIR, @"Directory containing help files", SXVTpath},
//   { @"", @"", @"", SXVT},
};
/** This must be the number of entries in 'DefaultSettings' */
static unsigned n_of_defaults =
	sizeof DefaultSettings / sizeof DefaultSettings[0];

@interface SXSettingsEntry: NSObject {
	NSString *	_key;
	NSString *	_value;
	NSString *	_description;
	// determines the behavior of the dialog
	int		_value_type;
	// widget allowing the user to set the parameter
	GTKWidget * _widget;
	GTKEntry  * _entry;
}
+entryWithKey: (NSString*) aKey 
	 value: (NSString*) aValue 
	 description: (NSString*) aDescription 
	 valueType: (int) value_type;  // either 'SXVTstring' or 'SXVTpath'
+entryFromString: (NSString*) descr;
-initWithKey: (NSString*) aKey 
	 value: (NSString*) aValue 
	 description: (NSString*) aDescription 
	 valueType: (int) value_type;  // either 'SXVTstring' or 'SXVTpath'
// init with description as created by -stringRepresentation
-initFromString: (NSString*) descr;
-(NSString*)key;
-(NSString*)value;
-(NSString*)description;
-(int)	valueType;
/** Returns a *new* widget allowing the use to set the parameter.
    The calling object is responsible for releasing the widget. */
-(GTKWidget*)widget;
/** Update the value with the current value of the widget. */
-(void)takeValueFromWidget;
/** Return a string representation of the contents.
    The result can be used to initialize a new SXSettingsEntry 
    using initFromString:
  */
-(NSString*)stringRepresentation;
@end

@interface SXSettings: NSObject {
	// file name of the file containing the settings
	NSString	* settings_file;
	// hashtable containing the settings
	NSMutableDictionary * dictionary;

	NSString	* col_separators;

	// settings dialog
	GTKDialog   * dialog;
}
/** The proper way to obtain the *unique* SXSettings-object: */
+(SXSettings*)settings;
-(BOOL)readSettingsFromFile: (NSString*) file_name;
// invokes writeSettingsToFile with the default file name
-(void)saveSettings;
-(BOOL)writeSettingsToFile:  (NSString*) file_name;
/** Set the default parameters */
-(void)setDefaults;
-(void)takeSettingsFromDialog;
// return number of entries in dictionary
-(unsigned)count;

/** Access hashtable: */
/** Returns the whole SXSettingsEntry object */
-(SXSettingsEntry*)objectForKey: aKey;
/** Returns the value containes in the according SXSettingsEntry 
    object as a string */
-(NSString*)valueForKey: aKey;
/** Add SXSettingsEntry object to dictionary. */
-(void)addObject: (SXSettingsEntry*)anObject;
-(void)removeObjectForKey: aKey;

/** Return the default separators for data files */
-(NSString*)columnSeparators;
/** return directory containing data files */
-(NSString*) dataDir;
// opens a sttings dialog
-(void)raiseSettingsDialog;


@end
#endif // Class_SXSettings
