/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/*	SaveDialog 
		- opens a save dialog, 
		- asks the user for a file name,
		- checks whether the file exists
		- and returns the file name or NULL if no file selected
 * Andreas Beyer, 1999
 */
#include "SaveDialog.h"

#include <Foundation/NSFileManager.h>
#include <Foundation/NSString.h>
#include <Foundation/NSException.h>

#include "Message.h"

@implementation SaveDialog

+(NSString*)runDialogWithInitialPath: (NSString*) path {
/** Run SaveDialog and return selected path on success or NULL otherwise.
    Checks whether selected file exists and asks user whether to overwrite
    it or not.
 */
	NSString* file_name;
	NSString* warning;
	NSFileManager* file_manager = [NSFileManager defaultManager];
	SaveDialog* dialog = [SaveDialog alloc];
	[dialog initSaveDialogWithInitialPath: path];
	if( !dialog ) [NSException raise: NSGenericException
		format: @"+runSaveDialogWithInitialPath: Can't create SaveDialog"];
	if( ![GTKApp runModalForWindow: dialog] ) {
		[dialog destroy];
		[dialog release];
		return NULL;
	}
	file_name = [dialog getFilename];
	warning = [NSString stringWithFormat: @"Warning: File\n%@\nexists", 
		[file_name lastPathComponent]];
	if( [file_manager fileExistsAtPath: file_name]
	&&	SXRunMessageBox (@"StatistX: Warning", warning,
			@"Overwrite", @"Cancel", nil) ) {
		file_name = NULL;
	}
	[dialog destroy];
	[dialog release];
	return [file_name retain];
}
+saveDialogWithInitialPath: (NSString*) path {
	return [ (SaveDialog*) [[self alloc] autorelease]
			initSaveDialogWithInitialPath: path];
}
-initSaveDialogWithInitialPath: (NSString*) path {
	[super initWithTitle:@"StatistX: Save"];
	[[self okButton] connectSignal:@"clicked" withTarget:self
		sel:@selector(stopWithTrue)];
    	[[self cancelButton] connectSignal:@"clicked" withTarget:self
		sel:@selector(stopWithFalse)];
	[self connectSignal:@"delete_event" withTarget:self
		sel:@selector(stopWithFalse)];
	[self showFileopButtons];
	if( path && [path length] > 0 ) [self setFilename: path];
	[self setPosition: GTK_WIN_POS_CENTER ];
	[self show];
	[[self okButton] grabFocus];
	return self;

}
-(void)stopWithTrue {
	[GTKApp stopModalWithCode: YES];
}
-(void)stopWithFalse {
	[GTKApp stopModalWithCode: NO];
}
@end
