/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/**	SelectColumnDialog.m - Dialog, that requests selection of a
 *    column by the user for a special action.
 *
 *	Andreas Beyer, 1999
 */
#include "SelectColumnDialog.h"
#include "Column.h"

#include <Foundation/NSObject.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSException.h>
#include <Foundation/NSString.h>
#include <Foundation/NSArray.h>

#include <GToolKit/GToolKit.h>

@implementation SelectColumnDialog
+(Column*)runDialogWithColumns: (NSArray*) _cols 
		message: (NSString*) message 
		title: (const NSString*) title {
	SelectColumnDialog* dialog =
		[[SelectColumnDialog alloc] initWithColumns: _cols 
			message: message title: title];
	if( ! dialog ) 	[NSException raise: NSGenericException
		format: @"+runSelectColumnDialogWithColumns: Can't create SelectColumnDialog: %s", title];
	if( [GTKApp runModalForWindow: dialog] ) {
		Column* result = [dialog getSelectedColumn];
		[dialog destroy];
		[dialog release];
		return result;
	}
	[dialog destroy];
	[dialog release];
	return NULL;
}
+dialogWithColumns: (NSArray*) _cols 
		message: (NSString*) message 
		title: (const NSString*) title {
	return [ (SelectColumnDialog*) [[self alloc] autorelease]
		initWithColumns: (NSArray*) _cols 
		message: (NSString*) message 
		title: (const NSString*) title];
}

-initWithColumns: (NSArray*) _cols 
		message: (NSString*) message 
		title: (const NSString*) title {
	GTKVBox *v_box;
	GTKHBox *buttons;
	GTKHBox *contents = [GTKHBox hBoxWithHomogeneous: NO spacing:0];
	GTKLabel* label = [GTKLabel labelWithStr: message];
	NSMutableArray* pop_down_strings;
	unsigned int i, n_cols;
	[super init];
	cols = [_cols retain];
	// make the two buttons
	ok_button = [GTKButton buttonWithLabel: @"OK"];
	[ok_button setUsize: 100 height: 30];
	[ok_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithTrue)]];
	cancel_button = [GTKButton buttonWithLabel: @"Cancel"];
	[cancel_button connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(stopWithFalse)]];
	[cancel_button setUsize: 100 height: 30];

	buttons = (GTKHBox*) [self actionArea];
	[buttons packStart:ok_button expand:NO fill:NO padding:0];
	[buttons packStart:cancel_button expand:NO fill:NO padding:0];

	// create the contents
	[label setPadding: 5 ypad: 10];
	combo = [GTKCombo combo];
	[combo setCaseSensitive: NO];
	n_cols = [cols count];
	pop_down_strings = [NSMutableArray arrayWithCapacity: n_cols];
	for(i = 0; i < n_cols; i++) {
		[pop_down_strings addObject: 
			[NSString stringWithCString: [(Column*)[cols objectAtIndex: i] getLabel]]];
	}
	//[combo setItemString:(GTKItem*) item itemValue:(const NSString *) item_value ];
	[combo setPopdownStrings: pop_down_strings];
	[contents packStart:label expand:YES fill:YES padding:0];
	[contents packStart:combo expand:YES fill:YES padding:0];
	
	v_box = (GTKVBox *) [self vbox];
	[v_box packStart:contents expand:YES fill:YES padding:0];
	[v_box setBorderWidth: 10];

	[self setTitle: [NSString stringWithCString: [title cString]]];
	[self setPosition: GTK_WIN_POS_CENTER ];

	[self show];
	[ok_button grabFocus];
	return self;
}
-(void)stopWithTrue {
	[GTKApp stopModalWithCode: YES];
}
-(void)stopWithFalse {
	[GTKApp stopModalWithCode: NO];
}

-(GTKButton*)okButton  {
	return ok_button;
}
-(GTKButton*)cancelButton {
	return cancel_button;
}
-(Column*) getSelectedColumn {
// Linear search.
// Is it worth creating a hash table for this function?
	char* c_name = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO([combo gtk])->entry));
//	NSString* text = [NSString stringWithCString: c_name];
	unsigned i, n_cols = [cols count];
	if( ! c_name || !*c_name ) return NULL;
	for(i = 0; i < n_cols; i++) {
		if(! strcmp(c_name, [(Column*)[cols objectAtIndex: i] getLabel]) )
			break;
	}
	if( i >= n_cols ) return NULL;
	return (Column*)[cols objectAtIndex: i];
}
-(void) dealloc {
	[cols release];
	[super dealloc];
}
@end
