/**	TextPanel.h - Dialog, that displays multi lined text.
 *	(E.g. statist results.)
 *
 *	Andreas Beyer, 1999
 */
 #include "TextPanel.h"
 #include "Message.h"
 #include "Manager.h"
 #include "SaveDialog.h"
 
#include <Foundation/NSObject.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSException.h>
#include <Foundation/NSString.h>
#include <Foundation/NSFileManager.h>
#include <Foundation/NSFileHandle.h>

#include <GToolKit/GToolKit.h>

@implementation TextPanel
+textPanelWithTitle: (const NSString*) title saveOption: (BOOL) save {
	return [ (TextPanel*) [[self alloc] autorelease]
		initWithTitle: title saveOption: save];

}
+textPanelWithTitle: (const NSString*) title {
	return [ (TextPanel*) [[self alloc] autorelease]
		initWithTitle: title];

}
-initWithTitle: (const NSString*) title {
	return [self initWithTitle: title saveOption: NO];
}
-initWithTitle: (const NSString*) title saveOption: (BOOL) allow_save {
	GTKButton* ok = [GTKButton buttonWithLabel: @"Close"];
	GTKButton* save = [GTKButton buttonWithLabel: @"Save"];
	GTKVBox*   v_box; // do not initialize before init
	GTKHBox*   h_box; // do not initialize before init
	GTKScrolledWindow* scroll_area
		= [GTKScrolledWindow scrolledWindowWithHadjustment: NULL vadjustment: NULL];
	GtkButton *gtk_button = [ok gtk];
	[super init];

	v_box = (GTKVBox*) [self vbox];
	h_box = (GTKHBox*) [self actionArea];
	
	if( allow_save ) {
		[save setUsize: 100 height: 30];
		[h_box packStart:save expand:YES fill:NO padding:0];
		gtk_misc_set_padding((GtkMisc *) gtk_button->child, 4, 0);
		[save connectSignal:@"clicked" withAction:
	        	[GTKAction actionWithTarget:self selector:@selector(saveText)]];
	}
	[ok setUsize: 100 height: 30];
	[h_box packStart:ok expand:YES fill:NO padding:0];
	gtk_misc_set_padding((GtkMisc *) gtk_button->child, 4, 0);
	[ok connectSignal:@"clicked" withAction:
        	[GTKAction actionWithTarget:self selector:@selector(destroy)]];

	text = [GTKText textWithHadj: NULL vadj: NULL];
	[text retain];
	[text setEditable: FALSE];
	[text setLineWrap: TRUE];
	[text setWordWrap: TRUE];
	[scroll_area add: text];
	[scroll_area setPolicy: GTK_POLICY_AUTOMATIC 
		vscrollbarPolicy:  GTK_POLICY_ALWAYS];
	[v_box packStart:scroll_area expand:YES fill:YES padding:0];
	[v_box setBorderWidth: 1];

	[self setTitle: title];
	[self setDefaultSize: 400 height: 200];
	[self setPosition: GTK_WIN_POS_CENTER ];
	[self show];
	[ok grabFocus];
	return self;
}
-(void)setEditable: (BOOL) editable {
	[text setEditable: editable];
}

-(void) appendString: (NSString*) string {
	// set cursor position:
	[text setPoint: [text getLength]];
	// insert the whole text:
	[text insert:NULL fore:NULL back:NULL chars:string length: [string length]];
}
-(void) clearText {
	unsigned l = [text getLength];
	[text setPoint: 0];
	[text forwardDelete:l];
}
-(NSString*) getText {
	return [text getChars: 0 endPos: -1];
}

-(void) saveText {
	NSData* 		contents;
	NSFileManager* 	file_manager; // needed to check access to file
	NSString* file_name = [SaveDialog runDialogWithInitialPath: nil];
	if( ! file_name ) return;
#ifdef DEBUG
	[(Manager*) GTKApp printOnDebug:
		[NSString stringWithFormat: @" saving text at %@ ...", file_name]];
#endif
	contents = [[self getText] dataUsingEncoding: 
					[NSString defaultCStringEncoding]];

	file_manager = [NSFileManager defaultManager];
 	if( ! [file_manager createFileAtPath:file_name
		contents:contents attributes: nil] ) {
		[Message messageWithContents: [NSString stringWithFormat: 
			@"Can't write to file %@", file_name]
			title: @"StatistX: Error"];
	}
}
-(void) dealloc {
	[text release];
	[super dealloc];
}
@end
