/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/* funcs.m - some functions which are required by StatisX
 *
 *
 * Andreas Beyer, 1999
 */
#include <stdlib.h>
#include <Foundation/NSException.h>

#include "sx_funcs.h"
#include "sx_defs.h"

void sx_clear_string( char* s, unsigned l ) {
	unsigned i;
	for( i = 0; i < l; i++ )
		s[i] = '\0';
}

char* sx_create_label( unsigned i, char* s ) {
/*  Create label for number 'i' and copy result in 's'. Returns NULL
    in case of an error. The number of characters used for the label is
    allway less then i+2. Except for i == 0 it is 2;
    Labels are of the form: A, B, C, ... Z, AA, AB, AC, ... AAA ... ZZZZZZZ
 */
 	unsigned m, l;
	if( ! s ) return NULL;
	
	if( i > 25 ) {
		m = i/26;
		i = i%26;
		s = sx_create_label(m, s);
		if( !s ) return NULL;
	}
	l = strlen( s );
	s[l]   = 'A'+(char)i;
	s[l+1] = '\0';
	return s;
}

void sx_copy_d_array( double* col1, const double* col2, unsigned n ) {
/** Copy col2 into col1. n = number of elements.*/
	unsigned i;
	for( i = 0; i < n; i++ ) {
		col1[i] = col2[i];
	}
}

// Allocate array for n number of double values
double* sx_calloc_double(unsigned n) {
	double *p = calloc( n, sizeof(*p) );
	if( !p ) [NSException raise: NSMallocException
			format: @"calloc_double: Not enough memory for data columns."];
	return p;
}

void sx_extend_array_length(double** arrays, unsigned n_arrays, unsigned length) {
	unsigned i, j, buf_offset = length/SX_BUF_LEN + 1;
	double *tmp;
	for( i = 0; i < n_arrays; i++ ) {
		if( !arrays[i] ) [NSException raise: NSGenericException
			format: @"extend_array_length: Array pointer no. %d contains null-pointer.", i];
		tmp = sx_calloc_double( buf_offset*SX_BUF_LEN );
		// copy data:
		for(j = 0; j < length; j++) tmp[j] = arrays[i][j];
		free( arrays[i] );
		arrays[i] = tmp;
	}
}

