/* This file is part of StatistX
**
** It is distributed under the GNU General Public License.
** See the file COPYING for details.
** 
** (c) Andreas Beyer, 1999 - 2000
** abeyer@usf.uni-osnabrueck.de
**
**  http://www.usf.Uni-Osnabrueck.DE/~abeyer/private/StatistX/
**  
***************************************************************/

/* sx_statist.m
   This file contains re-implementations of some of
   the original statist functions defined in statist.h

  Andreas Beyer, 1999
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <signal.h>

#include <Foundation/NSString.h>
#include <Foundation/NSObject.h>

#include "sx_statist.h"
#include "statist.h"

#include "Manager.h"
#include "TextPanel.h"
#include "Message.h"
#include "SXException.h"

typedef struct {
  char *optstr;
  BOOLEAN *optvar;
} OPTREC;


BOOLEAN get_options(char **args, int options[], int nopt, OPTREC optrec[]);
//void help_msg();

#ifndef MSDOS   
void handle_pipe_interrupt(int sig);
#endif
void handle_fpe_interrupt(int sig);
/* void handle_Ctrl_C_interrupt(int sig); */
void handle_term_interrupt(int sig);

PREAL   xx[MCOL], *yy, tempcol;
int     nn[MCOL], acol[MCOL], *ny, n_ptr=0;
BOOLEAN x_read[MCOL], empty, gnupl_open=FALSE;
int     ncol=0, status;
char    line[MLINE];
char    *alias[MCOL];
FILE    *logfile;
void    *ptr_arr[MPTR];

BOOLEAN silent=FALSE, noplot=FALSE, help=FALSE, log_set=FALSE,
        nofile=FALSE, nobell=FALSE, thist=FALSE, bernhard=FALSE,
        delrow=FALSE;

FILE    *pipef;

void initialize_statist() {
 	int i;
	line[0] = '\0';
	logfile = NULL;
	for( i = 0; i < MCOL; i++ ) {
		   nn[i] = acol[i] = 0;
	     x_read[i] = FALSE;
		alias[i] = mycalloc(1, sizeof(char) );
	}
}
/* ================================================================== */

@class PanelDelegate;
static TextPanel* current_panel;
static PanelDelegate* delegate = NULL;

// a class for the panel as delegate
@interface PanelDelegate: NSObject <GTKWindowDelegate>
- (void) windowDidBecomeMain:(NSNotification *) event;
- (void) windowDidResignMain:(NSNotification *) event;
- (BOOL) windowShouldClose:(GTKWindow *) sender;
- (void) windowWillClose:(NSNotification *) event;
@end
@implementation PanelDelegate 
- (void) windowDidBecomeMain:(NSNotification *) event {

}
- (void) windowDidResignMain:(NSNotification *) event {
}
- (BOOL) windowShouldClose:(GTKWindow *) sender {
	current_panel = NULL;
	return YES;
}
- (void) windowWillClose:(NSNotification *) event {
	current_panel = NULL;
}
@end

/* =================================================================== */

void finish() {
   erasetempfiles();
   if (log_set && logfile) {
     FCLOSE(logfile);
   }
#ifndef MSDOS
   if (gnupl_open) {
     fprintf(pipef, "quit\n");
     pclose(pipef);
   }
   if (myexist(GPL_DAT)) {
     remove(GPL_DAT);
   }
#endif
}


/* =================================================================== */

void mywait() {
;;
}

/* =================================================================== */

void *mycalloc(int nitems, int size) {
  void *pointer;
  pointer = calloc(nitems, size);
  if (pointer == NULL) {
     out_err(FAT, ERR_FILE, ERR_LINE, "Too few memory.");
     erasetempfiles();
	[NSException raise: NSMallocException
		format: @"statist: Memory allocation in statist-function."];
  }
  return pointer;
}


void *mymalloc(int size) {
  void *pointer;
  pointer = malloc(size);
  if (pointer == NULL) {
     out_err(FAT, ERR_FILE, ERR_LINE, "Too few memory.");
     erasetempfiles();
	[NSException raise: NSMallocException
		format: @"statist: Memory allocation in statist-function."];
  }
  return pointer;
}


/* =================================================================== */

void myfree(void *ptr) {
  free(ptr);
}

BOOLEAN myexist(char *name) {
   if (fopen(name,"r") == NULL) {
     return FALSE;
   }
   else {
     return TRUE;
   }
}

void out_start() {
	if( current_panel ) {
		[current_panel clearText];
	} else {
		if( !delegate ) delegate = [PanelDelegate new];
		current_panel = [[TextPanel alloc] initWithTitle: 
			@"StatistX: statist output" saveOption: YES];
		[current_panel setDelegate: delegate];
	}
}

void out_end() {
	if( current_panel ) {
		[current_panel appendString: @"\nDone."];
	//	[current_panel activateFocus];
	}
}
void out_r(char *fmt, ...) {
	va_list argptr;
	NSString* string = [NSString alloc];
	NSString* format = [NSString stringWithCString: fmt];

  va_start(argptr, fmt);
	string = [string initWithFormat: format arguments: argptr];
//	puts([string cString]);
	[current_panel appendString: string];
  if (log_set && logfile) {
    vfprintf(logfile, fmt, argptr);
  }
  va_end(argptr);
  [string release];
}


void out_d(char *fmt, ...) {
  va_list argptr;
  NSString* string = [NSString alloc];
  NSString* format = [NSString stringWithCString: fmt];

  if (!silent) {
    va_start(argptr, fmt);
	string = [string initWithFormat: format arguments: argptr];
//    puts([string cString]);
	[current_panel appendString: string];
    va_end(argptr);
  }
  [string release];
}



void out_err(int errno, char *modul, int lno, char *fmt, ...) {
  va_list argptr;
  int error;
  char where[80];
  NSString *format, *message = [NSString alloc];

  va_start(argptr, fmt);
  switch (errno) {
    case WAR: strcpy(line, "> statist: Warning ");
      break;
    case ERR: strcpy(line, "> statist: Error ");
      break;
    case FAT: strcpy(line, "\n > statist: Fatal error ");
      break;
    case MAT: strcpy(line, "> statist: Numerical error ");
      break;
    case MWA: strcpy(line, "> statist: Warning ");
      break;
  }

  error = errno;
  if (lno != 0) {
    sprintf(where, "(Modul %s, line %i):\n  ", modul, lno);
    strncat(line, where, 254-strlen(line));
  }
  else {
    strncat(line, ":\n  ", 254-strlen(line));
  }
  strncat(line, fmt, 254-strlen(line));
  if (nobell) {
    strncat(line, "\n\n", 254-strlen(line));
  }
  else {
    strncat(line, "\a\n\n", 254-strlen(line));
  }

//  vfprintf(stderr, line, argptr);
  format = [NSString stringWithCString: line];
  message = [message initWithFormat: format arguments: argptr];
  [Message messageWithContents: message title: @"statist: Warning"];
  [[Manager getManager] printOnDebug: message];
  if ( ((errno==MAT) || (errno==MWA)) && (log_set && logfile) ) {
    vfprintf(logfile, line, argptr);
  }
  va_end(argptr);
  [message release];
  if (error == FAT) {
	finish();
	[NSException raise: SXStatistFatalException
		format: @"statist: %s", line];
  }
}



int free_ptr(void *ptr_arr[], int n_ptr) {
  int i;
  for (i=0; i<n_ptr; i++) {
    myfree(ptr_arr[i]);
  }
  /* printf("%i Arrays geloescht\n", n_ptr);  */
  return 0;  
}  


void *m_calloc(int nitems, int size) {
  void *ptr;
  ptr = mycalloc(nitems, size);
  ptr_arr[n_ptr] = ptr;
  n_ptr++;
  if (n_ptr > MPTR) {
    out_err(FAT, ERR_FILE, ERR_LINE,
	    "More than %i dynamic variables allocated.", MPTR);
  }
  return ptr;
}

/* =================================================================== */

void print_histo(REAL x[], int y[], int n) {
  int maxclass;
  int   k, cols, i;
  char show[80];

  maxclass = get_maxint(y, n);
  for (i=0; i<n; i++) {
    cols = y[i]*60/maxclass;
    strcpy(show,"");
    for (k=0; k<cols; k++) {
      strcat(show, "*");
    }
    out_r("%9f %s %i\n", x[i], show, y[i]);
    if (((i+1) % SCREENSIZE) == 0) {
      mywait();
    }
  }
}


/* =================================================================== */

#ifndef MSDOS
void handle_pipe_interrupt(int sig) {
  gnupl_open = FALSE;
  out_err(WAR, ERR_FILE, ERR_LINE,
	  "Broken pipe to gnuplot. Creating a new one.\n");
  pclose(pipef);
}
#endif

void handle_fpe_interrupt(int sig) {
  out_err(FAT, ERR_FILE, ERR_LINE,
	  "Signal from operating system: Floating point exception.\n\
                      Division by 0?\n");
	[NSException raise: SXStatistException
		format: @"statist: Floating point exception."];
  
}

